@@ -1,29 +1,27 @@
 /*
  * Pound - the reverse-proxy load-balancer
- * Copyright (C) 2002-2006 Apsis GmbH
+ * Copyright (C) 2002-2010 Apsis GmbH
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- * Additionaly compiling, linking, and/or using OpenSSL is expressly allowed.
+ * This file is part of Pound.
  *
- * This program is distributed in the hope that it will be useful,
+ * Pound is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * Pound is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA  02111-1307, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  * Contact information:
  * Apsis GmbH
  * P.O.Box
  * 8707 Uetikon am See
  * Switzerland
- * Tel: +41-44-920 4904
  * EMail: roseg@apsis.ch
  */
 
@@ -33,12 +31,14 @@
 char        *user,              /* user to run as */
             *group,             /* group to run as */
             *root_jail,         /* directory to chroot to */
-            *pid_name;          /* file to record pid in */
+            *pid_name,          /* file to record pid in */
+            *ctrl_name;         /* control socket name */
 
 int         alive_to,           /* check interval for resurrection */
             daemonize,          /* run as daemon */
             log_facility,       /* log facility to use */
             print_log,          /* print log messages to stdout/stderr */
+            grace,              /* grace period before shutdown */
             control_sock;       /* control socket */
 
 SERVICE     *services;          /* global services (if any) */
@@ -49,10 +49,11 @@ regex_t HEADER,             /* Allowed header */
         CHUNK_HEAD,         /* chunk header line */
         RESP_SKIP,          /* responses for which we skip response */
         RESP_IGN,           /* responses for which we ignore content */
-        /* RESP_REDIR,         /* responses for which we rewrite Location */
         LOCATION,           /* the host we are redirected to */
         AUTHORIZATION;      /* the Authorisation header */
 
+static int  shut_down = 0;
+
 #ifndef  SOL_TCP
 /* for systems without the definition */
 int     SOL_TCP;
@@ -104,17 +105,41 @@ l_id(void)
 }
 
 /*
- * handle SIGTERM - exit
+ * handle SIGTERM/SIGQUIT - exit
  */
 static RETSIGTYPE
 h_term(const int sig)
 {
     logmsg(LOG_NOTICE, "received signal %d - exiting...", sig);
     if(son > 0)
-        kill(son, SIGTERM);
+        kill(son, sig);
+    if(ctrl_name != NULL)
+        (void)unlink(ctrl_name);
     exit(0);
 }
 
+/*
+ * handle SIGHUP/SIGINT - exit after grace period
+ */
+static RETSIGTYPE
+h_shut(const int sig)
+{
+    int         status;
+    LISTENER    *lstn;
+
+    logmsg(LOG_NOTICE, "received signal %d - shutting down...", sig);
+    if(son > 0) {
+        for(lstn = listeners; lstn; lstn = lstn->next)
+            close(lstn->sock);
+        kill(son, sig);
+        (void)wait(&status);
+        if(ctrl_name != NULL)
+            (void)unlink(ctrl_name);
+        exit(0);
+    } else
+        shut_down = 1;
+}
+
 /*
  * Pound: the reverse-proxy/load-balancer
  *
@@ -130,10 +155,11 @@ main(const int argc, char **argv)
     LISTENER            *lstn;
     pthread_t           thr;
     pthread_attr_t      attr;
+    struct sched_param  sp;
     uid_t               user_id;
     gid_t               group_id;
     FILE                *fpid;
-    struct sockaddr_in  clnt_addr;
+    struct sockaddr_storage clnt_addr;
     char                tmp[MAXBUF];
 #ifndef SOL_TCP
     struct protoent     *pe;
@@ -145,8 +171,9 @@ main(const int argc, char **argv)
     log_facility = -1;
     logmsg(LOG_NOTICE, "starting...");
 
+    signal(SIGHUP, h_shut);
+    signal(SIGINT, h_shut);
     signal(SIGTERM, h_term);
-    signal(SIGINT, h_term);
     signal(SIGQUIT, h_term);
     signal(SIGPIPE, SIG_IGN);
 
@@ -166,7 +193,6 @@ main(const int argc, char **argv)
     || regcomp(&CHUNK_HEAD, "^([0-9a-f]+).*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&RESP_SKIP, "^HTTP/1.1 100.*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&RESP_IGN, "^HTTP/1.[01] (10[1-9]|1[1-9][0-9]|204|30[456]).*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
-    /* || regcomp(&RESP_REDIR, "^HTTP/1.[01] 30[1237].*$", REG_ICASE | REG_NEWLINE | REG_EXTENDED) */
     || regcomp(&LOCATION, "(http|https)://([^/]+)(.*)", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     || regcomp(&AUTHORIZATION, "Authorization:[ \t]*Basic[ \t]*([^ \t]*)[ \t]*", REG_ICASE | REG_NEWLINE | REG_EXTENDED)
     ) {
@@ -188,24 +214,39 @@ main(const int argc, char **argv)
 
     if(log_facility != -1)
         openlog("pound", LOG_CONS | LOG_NDELAY, LOG_DAEMON);
+    if(ctrl_name != NULL) {
+        struct sockaddr_un  ctrl;
 
-    /* open HTTP listeners */
+        memset(&ctrl, 0, sizeof(ctrl));
+        ctrl.sun_family = AF_UNIX;
+        strncpy(ctrl.sun_path, ctrl_name, sizeof(ctrl.sun_path) - 1);
+        (void)unlink(ctrl.sun_path);
+        if((control_sock = socket(PF_UNIX, SOCK_STREAM, 0)) < 0) {
+            logmsg(LOG_ERR, "Control \"%s\" create: %s", ctrl.sun_path, strerror(errno));
+            exit(1);
+        }
+        if(bind(control_sock, (struct sockaddr *)&ctrl, (socklen_t)sizeof(ctrl)) < 0) {
+            logmsg(LOG_ERR, "Control \"%s\" bind: %s", ctrl.sun_path, strerror(errno));
+            exit(1);
+        }
+        listen(control_sock, 512);
+    }
+
+    /* open listeners */
     for(lstn = listeners, n_listeners = 0; lstn; lstn = lstn->next, n_listeners++) {
         int opt;
 
         /* prepare the socket */
-        if((lstn->sock = socket(PF_INET, SOCK_STREAM, 0)) < 0) {
-            addr2str(tmp, MAXBUF - 1, &lstn->addr.sin_addr);
-            logmsg(LOG_ERR, "HTTP socket %s:%hd create: %s - aborted",
-                tmp, ntohs(lstn->addr.sin_port), strerror(errno));
+        if((lstn->sock = socket(lstn->addr.ai_family == AF_INET? PF_INET: PF_INET6, SOCK_STREAM, 0)) < 0) {
+            addr2str(tmp, MAXBUF - 1, &lstn->addr, 0);
+            logmsg(LOG_ERR, "HTTP socket %s create: %s - aborted", tmp, strerror(errno));
             exit(1);
         }
         opt = 1;
         setsockopt(lstn->sock, SOL_SOCKET, SO_REUSEADDR, (void *)&opt, sizeof(opt));
-        if(bind(lstn->sock, (struct sockaddr *)&lstn->addr, (socklen_t)sizeof(lstn->addr)) < 0) {
-            addr2str(tmp, MAXBUF - 1, &lstn->addr.sin_addr);
-            logmsg(LOG_ERR, "HTTP socket bind %s:%hd: %s - aborted",
-                tmp, ntohs(lstn->addr.sin_port), strerror(errno));
+        if(bind(lstn->sock, lstn->addr.ai_addr, (socklen_t)lstn->addr.ai_addrlen) < 0) {
+            addr2str(tmp, MAXBUF - 1, &lstn->addr, 0);
+            logmsg(LOG_ERR, "HTTP socket bind %s: %s - aborted", tmp, strerror(errno));
             exit(1);
         }
         listen(lstn->sock, 512);
@@ -301,8 +342,7 @@ main(const int argc, char **argv)
         if((son = fork()) > 0) {
             int status;
 
-            while(wait(&status) != son)
-                logmsg(LOG_ERR, "MONITOR: bad wait (%s)", strerror(errno));
+            (void)wait(&status);
             if(WIFEXITED(status))
                 logmsg(LOG_ERR, "MONITOR: worker exited normally %d, restarting...", WEXITSTATUS(status));
             else if(WIFSIGNALED(status))
@@ -311,6 +351,7 @@ main(const int argc, char **argv)
                 logmsg(LOG_ERR, "MONITOR: worker exited (stopped?) %d, restarting...", status);
         } else if (son == 0) {
 #endif
+
             /* thread stuff */
             pthread_attr_init(&attr);
             pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
@@ -339,6 +380,18 @@ main(const int argc, char **argv)
 
             /* and start working */
             for(;;) {
+                if(shut_down) {
+                    logmsg(LOG_NOTICE, "shutting down...");
+                    for(lstn = listeners; lstn; lstn = lstn->next)
+                        close(lstn->sock);
+                    if(grace > 0) {
+                        sleep(grace);
+                        logmsg(LOG_NOTICE, "grace period expired - exiting...");
+                    }
+                    if(ctrl_name != NULL)
+                        (void)unlink(ctrl_name);
+                    exit(0);
+                }
                 for(lstn = listeners, i = 0; i < n_listeners; lstn = lstn->next, i++) {
                     polls[i].events = POLLIN | POLLPRI;
                     polls[i].revents = 0;
@@ -353,32 +406,45 @@ main(const int argc, char **argv)
                             if((clnt = accept(lstn->sock, (struct sockaddr *)&clnt_addr,
                                 (socklen_t *)&clnt_length)) < 0) {
                                 logmsg(LOG_WARNING, "HTTP accept: %s", strerror(errno));
-                            } else if (clnt_addr.sin_family != AF_INET) {
-                                /* may happen on FreeBSD, I am told */
-                                logmsg(LOG_WARNING, "HTTP connection prematurely closed by peer");
-                                close(clnt);
-                            } else if(lstn->disabled) {
-                                /*
-                                addr2str(tmp, MAXBUF - 1, &clnt_addr.sin_addr);
-                                logmsg(LOG_WARNING, "HTTP disabled listener from %s", tmp);
-                                */
-                                close(clnt);
-                            } else {
+                            } else if(((struct sockaddr_in *)&clnt_addr)->sin_family == AF_INET
+                                   || ((struct sockaddr_in *)&clnt_addr)->sin_family == AF_INET6) {
                                 thr_arg *arg;
 
+                                if(lstn->disabled) {
+                                    /*
+                                    addr2str(tmp, MAXBUF - 1, &clnt_addr, 1);
+                                    logmsg(LOG_WARNING, "HTTP disabled listener from %s", tmp);
+                                    */
+                                    close(clnt);
+                                }
                                 if((arg = (thr_arg *)malloc(sizeof(thr_arg))) == NULL) {
                                     logmsg(LOG_WARNING, "HTTP arg: malloc");
                                     close(clnt);
-                                } else {
-                                    arg->sock = clnt;
-                                    arg->lstn = lstn;
-                                    arg->from_host = clnt_addr.sin_addr;
-                                    if(pthread_create(&thr, &attr, thr_http, (void *)arg)) {
-                                        logmsg(LOG_WARNING, "HTTP pthread_create: %s", strerror(errno));
-                                        free(arg);
-                                        close(clnt);
-                                    }
+                                    continue;
                                 }
+                                arg->sock = clnt;
+                                arg->lstn = lstn;
+                                if((arg->from_host.ai_addr = (struct sockaddr *)malloc(clnt_length)) == NULL) {
+                                    logmsg(LOG_WARNING, "HTTP arg address: malloc");
+                                    free(arg);
+                                    continue;
+                                }
+                                memcpy(arg->from_host.ai_addr, &clnt_addr, clnt_length);
+                                arg->from_host.ai_addrlen = clnt_length;
+                                if(((struct sockaddr_in *)&clnt_addr)->sin_family == AF_INET)
+                                    arg->from_host.ai_family = AF_INET;
+                                else
+                                    arg->from_host.ai_family = AF_INET6;
+                                if(pthread_create(&thr, &attr, thr_http, (void *)arg)) {
+                                    logmsg(LOG_WARNING, "HTTP pthread_create: %s", strerror(errno));
+                                    free(arg->from_host.ai_addr);
+                                    free(arg);
+                                    close(clnt);
+                                }
+                            } else {
+                                /* may happen on FreeBSD, I am told */
+                                logmsg(LOG_WARNING, "HTTP connection prematurely closed by peer");
+                                close(clnt);
                             }
                         }
                     }
